/*****************************************************************************************************************************
--
--    File Name    : FM_Median_Filter.v 

--    Description  : This module implements FM_Median_Filter .

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

module FM_Median_Filter #(parameter g_DATAWIDTH = 24,
                          parameter g_X_RES_WIDTH = 11,
                          parameter g_DISPLAY_RESOLUTION = 1280,
                          parameter g_VERT_DISPLAY_RESOLUTION = 720)
                        (input wire SYS_CLK_I,
                         input wire RESET_n_I,
                         input wire[g_DATAWIDTH - 1 : 0] Data_In_i,
                         input wire Data_In_Vld_i,
                         output reg[(g_DATAWIDTH / 3) - 1 : 0] R_Out_o,
                         output reg[(g_DATAWIDTH / 3) - 1 : 0] G_Out_o,
                         output reg[(g_DATAWIDTH / 3) - 1 : 0] B_Out_o,
                         output reg RGB_Vld_out_o);
  parameter[2 : 0] ll = 3'd0,
                   ol = 3'd1,
                   il = 3'd2,
                   O0 = 3'd3,
                   I0 = 3'd4,
                   l0 = 3'd5;
  parameter[0 : 0] Ol = 1'd0,
                   Il = 1'd1;
  reg[g_X_RES_WIDTH - 1 : 0] o0;
  reg[g_DATAWIDTH - 1 : 0] i0;
  reg[2 : 0] O1;
  reg I1;
  reg l1;
  reg o1;
  reg i1;
  reg Oo;
  reg Io;
  reg lo;
  reg oo;
  reg io;
  reg Oi;
  reg Ii;
  reg[g_X_RES_WIDTH - 1 : 0] li;
  reg[g_X_RES_WIDTH - 1 : 0] oi;
  reg[g_X_RES_WIDTH - 1 : 0] ii;
  reg[g_X_RES_WIDTH - 1 : 0] OOI;
  reg[g_X_RES_WIDTH - 1 : 0] IOI;
  reg[g_DATAWIDTH - 1 : 0] lOI;
  reg[g_DATAWIDTH - 1 : 0] oOI;
  reg[g_DATAWIDTH - 1 : 0] iOI;
  reg[g_DATAWIDTH - 1 : 0] OII;
  reg[g_DATAWIDTH - 1 : 0] III;
  reg[g_DATAWIDTH - 1 : 0] lII;
  reg[g_DATAWIDTH - 1 : 0] oII;
  reg[g_DATAWIDTH - 1 : 0] iII;
  reg[g_DATAWIDTH - 1 : 0] OlI;
  reg[g_DATAWIDTH - 1 : 0] IlI;
  reg[g_DATAWIDTH - 1 : 0] llI;
  reg[g_DATAWIDTH - 1 : 0] olI;
  reg signed[(g_DATAWIDTH / 3) : 0] I1l;
  reg signed[(g_DATAWIDTH / 3) : 0] l1l;
  reg signed[(g_DATAWIDTH / 3) : 0] o1l;
  reg signed[(g_DATAWIDTH / 3) : 0] i1l;
  reg signed[(g_DATAWIDTH / 3) : 0] Ool;
  reg signed[(g_DATAWIDTH / 3) : 0] Iol;
  reg signed[(g_DATAWIDTH / 3) : 0] lol;
  reg signed[(g_DATAWIDTH / 3) : 0] ool;
  reg signed[(g_DATAWIDTH / 3) : 0] iol;
  reg signed[(g_DATAWIDTH / 3) : 0] Oil;
  reg signed[(g_DATAWIDTH / 3) : 0] Iil;
  reg signed[(g_DATAWIDTH / 3) : 0] lil;
  reg signed[(g_DATAWIDTH / 3) : 0] oil;
  reg signed[(g_DATAWIDTH / 3) : 0] iil;
  reg signed[(g_DATAWIDTH / 3) : 0] OO0;
  reg signed[(g_DATAWIDTH / 3) : 0] IO0;
  reg signed[(g_DATAWIDTH / 3) : 0] lO0;
  reg signed[(g_DATAWIDTH / 3) : 0] oO0;
  reg[g_X_RES_WIDTH - 1 : 0] o1I;
  reg[g_X_RES_WIDTH - 1 : 0] i1I;
  reg[2 : 0] IoI;
  reg[g_DATAWIDTH / 3 + 2 : 0] I;
  reg[g_DATAWIDTH / 3 + 2 : 0] l;
  reg[g_DATAWIDTH / 3 + 2 : 0] o;
  reg[g_DATAWIDTH / 3 + 2 : 0] Ill;
  reg[g_DATAWIDTH / 3 + 2 : 0] lll;
  reg[g_DATAWIDTH / 3 + 2 : 0] oll;
  reg[g_DATAWIDTH / 3 - 1 : 0] iO0;
  reg[2 : 0] oiI;
  reg iiI;
  reg OOl;
  reg IOl;
  reg[g_X_RES_WIDTH - 1 : 0] lOl;
  reg oOl;
  reg OiI;
  reg[g_X_RES_WIDTH - 1 : 0] IiI;
  reg[0 : 0] liI;
  wire[g_DATAWIDTH - 1 : 0] OIl;
  wire[g_DATAWIDTH - 1 : 0] IIl;
  wire[g_DATAWIDTH - 1 : 0] lIl;
  wire[g_DATAWIDTH - 1 : 0] oIl;
  wire[g_DATAWIDTH - 1 : 0] iIl;
  wire signed[(g_DATAWIDTH / 3) : 0] OI0;
  wire signed[(g_DATAWIDTH / 3) : 0] II0;
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          o1I <= { g_X_RES_WIDTH { 1'b1 } };
        end
      else
        begin
          if (!I1 || o1I > g_DISPLAY_RESOLUTION - 1)
            begin
              o1I <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            if (I1)
              begin
                if (iiI)
                  o1I <= { g_X_RES_WIDTH { 1'b0 } };
                else
                  o1I <= o1I + 1'b1;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          i1I <= { g_X_RES_WIDTH { 1'b1 } };
        end
      else
        begin
          if (!I1)
            begin
              i1I <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (o1I > g_DISPLAY_RESOLUTION - 1 && I1)
                begin
                  i1I <= i1I + 12'd1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          o0 <= { g_X_RES_WIDTH { 1'b1 } };
        end
      else
        begin
          if (o0 == g_DISPLAY_RESOLUTION - 1)
            begin
              o0 <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (Data_In_Vld_i == 1'b1)
                begin
                  o0 <= o0 + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          O1 <= 3'd0;
        end
      else
        begin
          if ((O1 == 3'd4 && o0 == g_DISPLAY_RESOLUTION - 1) || (lOl == g_VERT_DISPLAY_RESOLUTION - 1 && o0 == g_DISPLAY_RESOLUTION - 1))
            begin
              O1 <= 3'd0;
            end
          else
            begin
              if (o0 == g_DISPLAY_RESOLUTION - 1)
                begin
                  O1 <= O1 + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          lOl <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if (lOl == g_VERT_DISPLAY_RESOLUTION)
            begin
              lOl <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (o0 == g_DISPLAY_RESOLUTION - 1 && oOl == 1'b1)
                begin
                  lOl <= lOl + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          OOl <= 1'b0;
        end
      else
        begin
          if (lOl == g_VERT_DISPLAY_RESOLUTION)
            begin
              OOl <= 1'b1;
            end
          else
            if (IOl)
              begin
                OOl <= 1'b0;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          i0 <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          i0 <= Data_In_i;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          oOl <= 1'b0;
        end
      else
        begin
          oOl <= Data_In_Vld_i;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          I1 <= 1'b0;
        end
      else
        begin
          if (O1 >= 3'd2 && o0 >= 11'd4)
            begin
              I1 <= 1'b1;
            end
          else
            if ((i1I == g_VERT_DISPLAY_RESOLUTION - 1) && (o1I == g_DISPLAY_RESOLUTION - 1))
              begin
                I1 <= 1'b0;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          oiI <= ll;
        end
      else
        begin
          case (oiI)
            ll:
              begin
                if (O1 >= 3'd2 && o0 >= 11'd4)
                  begin
                    oiI <= ol;
                  end
                else
                  begin
                    oiI <= ll;
                  end
              end
            ol:
              begin
                if (o1I == g_DISPLAY_RESOLUTION)
                  begin
                    oiI <= l0;
                  end
                else
                  begin
                    oiI <= ol;
                  end
              end
            l0:
              begin
                if (OOl)
                  begin
                    oiI <= O0;
                  end
                else
                  begin
                    oiI <= il;
                  end
              end
            il:
              begin
                if (Data_In_Vld_i && o0 >= 11'd4)
                  begin
                    oiI <= O0;
                  end
                else
                  begin
                    oiI <= il;
                  end
              end
            O0:
              begin
                if (i1I == g_VERT_DISPLAY_RESOLUTION - 1)
                  begin
                    oiI <= I0;
                  end
                else
                  begin
                    oiI <= ol;
                  end
              end
            I0:
              begin
                if (o1I == g_DISPLAY_RESOLUTION - 1)
                  begin
                    oiI <= ll;
                  end
                else
                  begin
                    oiI <= I0;
                  end
              end
            default:
              oiI <= ll;
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          iiI <= 1'b0;
          IOl <= 1'b0;
        end
      else
        begin
          case (oiI)
            ll:
              begin
                iiI <= 1'b0;
                IOl <= 1'b0;
              end
            ol:
              begin
                if (o1I == g_DISPLAY_RESOLUTION)
                  begin
                    iiI <= 1'b1;
                  end
                else
                  begin
                    iiI <= 1'b0;
                  end
              end
            l0:
              begin
                iiI <= 1'b1;
              end
            il:
              begin
                if (Data_In_Vld_i && o0 >= 11'd4)
                  begin
                    iiI <= 1'b0;
                  end
                else
                  begin
                    iiI <= iiI;
                  end
              end
            O0:
              begin
                iiI <= 1'b0;
              end
            I0:
              begin
                if (o1I == g_DISPLAY_RESOLUTION - 1)
                  begin
                    iiI <= 1'b0;
                    IOl <= 1'b1;
                  end
                else
                  begin
                    iiI <= iiI;
                    IOl <= 1'b0;
                  end
              end
            default:
              ;
          endcase
        end
    end
  always
    @(*)
    begin
      if (!RESET_n_I)
        begin
          l1 = 1'b0;
          o1 = 1'b0;
          i1 = 1'b0;
          Oo = 1'b0;
          Io = 1'b0;
        end
      else
        begin
          case (O1)
            3'd0:
              begin
                l1 = 1'b1;
                o1 = 1'b0;
                i1 = 1'b0;
                Oo = 1'b0;
                Io = 1'b0;
              end
            3'd1:
              begin
                l1 = 1'b0;
                o1 = 1'b1;
                i1 = 1'b0;
                Oo = 1'b0;
                Io = 1'b0;
              end
            3'd2:
              begin
                l1 = 1'b0;
                o1 = 1'b0;
                i1 = 1'b1;
                Oo = 1'b0;
                Io = 1'b0;
              end
            3'd3:
              begin
                l1 = 1'b0;
                o1 = 1'b0;
                i1 = 1'b0;
                Oo = 1'b1;
                Io = 1'b0;
              end
            3'd4:
              begin
                l1 = 1'b0;
                o1 = 1'b0;
                i1 = 1'b0;
                Oo = 1'b0;
                Io = 1'b1;
              end
            default:
              begin
                l1 = 1'b0;
                o1 = 1'b0;
                i1 = 1'b0;
                Oo = 1'b0;
                Io = 1'b0;
              end
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          IoI <= 3'd0;
        end
      else
        begin
          if (!I1 || (IoI == 3'd4 && o1I == g_DISPLAY_RESOLUTION))
            begin
              IoI <= 3'd0;
            end
          else
            begin
              if (o1I >= g_DISPLAY_RESOLUTION && I1)
                begin
                  IoI <= IoI + 12'd1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          lo <= 1'b1;
          oo <= 1'b1;
          io <= 1'b1;
          Oi <= 1'b1;
          Ii <= 1'b1;
        end
      else
        begin
          case (IoI)
            3'd0:
              begin
                lo <= 1'b0;
                oo <= 1'b0;
                io <= 1'b0;
                Oi <= 1'b1;
                Ii <= 1'b1;
              end
            3'd1:
              begin
                lo <= 1'b1;
                oo <= 1'b0;
                io <= 1'b0;
                Oi <= 1'b0;
                Ii <= 1'b1;
              end
            3'd2:
              begin
                lo <= 1'b1;
                oo <= 1'b1;
                io <= 1'b0;
                Oi <= 1'b0;
                Ii <= 1'b0;
              end
            3'd3:
              begin
                lo <= 1'b0;
                oo <= 1'b1;
                io <= 1'b1;
                Oi <= 1'b0;
                Ii <= 1'b0;
              end
            3'd4:
              begin
                lo <= 1'b0;
                oo <= 1'b0;
                io <= 1'b1;
                Oi <= 1'b1;
                Ii <= 1'b0;
              end
            default:
              ;
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          li <= { g_X_RES_WIDTH { 1'b0 } };
          oi <= { g_X_RES_WIDTH { 1'b0 } };
          ii <= { g_X_RES_WIDTH { 1'b0 } };
          OOI <= { g_X_RES_WIDTH { 1'b0 } };
          IOI <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if ((I1 && (li > g_DISPLAY_RESOLUTION - 1 || oi > g_DISPLAY_RESOLUTION - 1 || ii > g_DISPLAY_RESOLUTION - 1 || OOI > g_DISPLAY_RESOLUTION - 1 || IOI > g_DISPLAY_RESOLUTION - 1)))
            begin
              li <= { g_X_RES_WIDTH { 1'b0 } };
              oi <= { g_X_RES_WIDTH { 1'b0 } };
              ii <= { g_X_RES_WIDTH { 1'b0 } };
              OOI <= { g_X_RES_WIDTH { 1'b0 } };
              IOI <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (I1)
                begin
                  if (iiI == 1'b1)
                    begin
                      li <= li;
                      oi <= oi;
                      ii <= ii;
                      OOI <= OOI;
                      IOI <= IOI;
                    end
                  else
                    begin
                      case (IoI)
                        3'd0:
                          begin
                            li <= li + 1'b1;
                            oi <= oi + 1'b1;
                            ii <= ii + 1'b1;
                          end
                        3'd1:
                          begin
                            oi <= oi + 1'b1;
                            ii <= ii + 1'b1;
                            OOI <= OOI + 1'b1;
                          end
                        3'd2:
                          begin
                            ii <= ii + 1'b1;
                            OOI <= OOI + 1'b1;
                            IOI <= IOI + 1'b1;
                          end
                        3'd3:
                          begin
                            OOI <= OOI + 1'b1;
                            IOI <= IOI + 1'b1;
                            li <= li + 1'b1;
                          end
                        3'd4:
                          begin
                            IOI <= IOI + 1'b1;
                            li <= li + 1'b1;
                            oi <= oi + 1'b1;
                          end
                        default:
                          ;
                      endcase
                    end
                end
              else
                if (!I1)
                  begin
                    li <= { g_X_RES_WIDTH { 1'b0 } };
                    oi <= { g_X_RES_WIDTH { 1'b0 } };
                    ii <= { g_X_RES_WIDTH { 1'b0 } };
                    OOI <= { g_X_RES_WIDTH { 1'b0 } };
                    IOI <= { g_X_RES_WIDTH { 1'b0 } };
                  end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          lOI <= { g_DATAWIDTH { 1'b0 } };
          III <= { g_DATAWIDTH { 1'b0 } };
          OlI <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (ii == 11'd1 || OOI == 11'd1 || IOI == 11'd1)
            begin
              case (IoI)
                3'd0:
                  begin
                    lOI <= OIl;
                    III <= IIl;
                    OlI <= lIl;
                  end
                3'd1:
                  begin
                    lOI <= IIl;
                    III <= lIl;
                    OlI <= oIl;
                  end
                3'd2:
                  begin
                    lOI <= lIl;
                    III <= oIl;
                    OlI <= iIl;
                  end
                3'd3:
                  begin
                    lOI <= oIl;
                    III <= iIl;
                    OlI <= OIl;
                  end
                3'd4:
                  begin
                    lOI <= iIl;
                    III <= OIl;
                    OlI <= IIl;
                  end
                default:
                  ;
              endcase
            end
          else
            begin
              if (ii >= 11'd4 || OOI >= 11'd4 || IOI >= 11'd4)
                begin
                  lOI <= oOI;
                  III <= lII;
                  OlI <= IlI;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          oOI <= { g_DATAWIDTH { 1'b0 } };
          lII <= { g_DATAWIDTH { 1'b0 } };
          IlI <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (ii == 11'd2 || OOI == 11'd2 || IOI == 11'd2)
            begin
              case (IoI)
                3'd0:
                  begin
                    oOI <= OIl;
                    lII <= IIl;
                    IlI <= lIl;
                  end
                3'd1:
                  begin
                    oOI <= IIl;
                    lII <= lIl;
                    IlI <= oIl;
                  end
                3'd2:
                  begin
                    oOI <= lIl;
                    lII <= oIl;
                    IlI <= iIl;
                  end
                3'd3:
                  begin
                    oOI <= oIl;
                    lII <= iIl;
                    IlI <= OIl;
                  end
                3'd4:
                  begin
                    oOI <= iIl;
                    lII <= OIl;
                    IlI <= IIl;
                  end
                default:
                  ;
              endcase
            end
          else
            begin
              if (ii >= 11'd4 || OOI >= 11'd4 || IOI >= 11'd4)
                begin
                  oOI <= iOI;
                  lII <= oII;
                  IlI <= llI;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          iOI <= { g_DATAWIDTH { 1'b0 } };
          oII <= { g_DATAWIDTH { 1'b0 } };
          llI <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (ii == 11'd3 || OOI == 11'd3 || IOI == 11'd3)
            begin
              case (IoI)
                3'd0:
                  begin
                    iOI <= OIl;
                    oII <= IIl;
                    llI <= lIl;
                  end
                3'd1:
                  begin
                    iOI <= IIl;
                    oII <= lIl;
                    llI <= oIl;
                  end
                3'd2:
                  begin
                    iOI <= lIl;
                    oII <= oIl;
                    llI <= iIl;
                  end
                3'd3:
                  begin
                    iOI <= oIl;
                    oII <= iIl;
                    llI <= OIl;
                  end
                3'd4:
                  begin
                    iOI <= iIl;
                    oII <= OIl;
                    llI <= IIl;
                  end
                default:
                  ;
              endcase
            end
          else
            begin
              if (ii >= 11'd4 || OOI >= 11'd4 || IOI >= 11'd4)
                begin
                  iOI <= OII;
                  oII <= iII;
                  llI <= olI;
                end
            end
        end
    end
  always
    @(*)
    begin
      if (ii >= 11'd4 || OOI >= 11'd4 || IOI >= 11'd4)
        begin
          case (IoI)
            3'd0:
              begin
                OII = OIl;
                iII = IIl;
                olI = lIl;
              end
            3'd1:
              begin
                OII = IIl;
                iII = lIl;
                olI = oIl;
              end
            3'd2:
              begin
                OII = lIl;
                iII = oIl;
                olI = iIl;
              end
            3'd3:
              begin
                OII = oIl;
                iII = iIl;
                olI = OIl;
              end
            3'd4:
              begin
                OII = iIl;
                iII = OIl;
                olI = IIl;
              end
            default:
              begin
                OII = iIl;
                iII = OIl;
                olI = IIl;
              end
          endcase
        end
      else
        begin
          OII = { g_DATAWIDTH { 1'b0 } };
          iII = { g_DATAWIDTH { 1'b0 } };
          olI = { g_DATAWIDTH { 1'b0 } };
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          OiI <= 1'b0;
        end
      else
        begin
          if (li == g_DISPLAY_RESOLUTION || oi == g_DISPLAY_RESOLUTION || ii == g_DISPLAY_RESOLUTION || OOI == g_DISPLAY_RESOLUTION || IOI == g_DISPLAY_RESOLUTION)
            begin
              OiI <= 1'b1;
            end
          else
            if (li <= 11'd4 || oi >= 11'd4 || ii >= 11'd4 || OOI >= 11'd4 || IOI >= 11'd4)
              begin
                OiI <= 1'b0;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          I1l <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          l1l <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          o1l <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          i1l <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          Ool <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          Iol <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          lol <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          ool <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          iol <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
        end
      else
        begin
          if (ii >= 11'd4 || OOI >= 11'd4 || IOI >= 11'd4 || oi >= 11'd4 || li >= 11'd4 || OiI)
            begin
              I1l <= (lOI[23 : 16] - lOI[15 : 8]);
              l1l <= (oOI[23 : 16] - oOI[15 : 8]);
              o1l <= (iOI[23 : 16] - iOI[15 : 8]);
              i1l <= (III[23 : 16] - III[15 : 8]);
              Ool <= (lII[23 : 16] - lII[15 : 8]);
              Iol <= (oII[23 : 16] - oII[15 : 8]);
              lol <= (OlI[23 : 16] - OlI[15 : 8]);
              ool <= (IlI[23 : 16] - IlI[15 : 8]);
              iol <= (llI[23 : 16] - llI[15 : 8]);
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          Oil <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          Iil <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          lil <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          oil <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          iil <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          OO0 <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          IO0 <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          lO0 <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          oO0 <= { (g_DATAWIDTH / 3 + 1) { 1'b0 } };
          iO0 <= { (g_DATAWIDTH / 3) { 1'b0 } };
        end
      else
        begin
          if (ii >= 11'd4 || OOI >= 11'd4 || IOI >= 11'd4 || oi >= 11'd4 || li >= 11'd4 || OiI)
            begin
              Oil <= (lOI[7 : 0] - lOI[15 : 8]);
              Iil <= (oOI[7 : 0] - oOI[15 : 8]);
              lil <= (iOI[7 : 0] - iOI[15 : 8]);
              oil <= (III[7 : 0] - III[15 : 8]);
              iil <= (lII[7 : 0] - lII[15 : 8]);
              OO0 <= (oII[7 : 0] - oII[15 : 8]);
              IO0 <= (OlI[7 : 0] - OlI[15 : 8]);
              lO0 <= (IlI[7 : 0] - IlI[15 : 8]);
              oO0 <= (llI[7 : 0] - llI[15 : 8]);
              iO0 <= lII[15 : 8];
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          liI <= Ol;
          IiI <= 11'd0;
          RGB_Vld_out_o <= 1'b0;
        end
      else
        begin
          case (liI)
            Ol:
              begin
                if (o1I == 11'd7)
                  begin
                    IiI <= IiI + 1'b1;
                    RGB_Vld_out_o <= 1'b1;
                    liI <= Il;
                  end
                else
                  begin
                    liI <= Ol;
                    IiI <= 11'd0;
                    RGB_Vld_out_o <= 1'b0;
                  end
              end
            Il:
              begin
                if (IiI == g_DISPLAY_RESOLUTION)
                  begin
                    liI <= Ol;
                    IiI <= 11'd0;
                    RGB_Vld_out_o <= 1'b0;
                  end
                else
                  begin
                    IiI <= IiI + 1'b1;
                    RGB_Vld_out_o <= 1'b1;
                    liI <= Il;
                  end
              end
            default:
              ;
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          Ill <= 11'd0;
          lll <= 11'd0;
          oll <= 11'd0;
        end
      else
        begin
          Ill <= ($signed(OI0) + $signed({ 2'b00, iO0 }));
          lll <= { 3'b000, iO0 };
          oll <= ($signed(II0) + $signed({ 2'b00, iO0 }));
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          I <= 11'd0;
          l <= 11'd0;
          o <= 11'd0;
        end
      else
        begin
          I <= Ill;
          l <= lll;
          o <= oll;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          R_Out_o <= 8'd0;
        end
      else
        begin
          if ($signed(I) < $signed(11'd0))
            begin
              R_Out_o <= 8'd0;
            end
          else
            begin
              if ($signed(I) > $signed(11'd255))
                begin
                  R_Out_o <= 8'd255;
                end
              else
                begin
                  R_Out_o <= I[7 : 0];
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          G_Out_o <= 8'd0;
        end
      else
        begin
          if ($signed(l) < $signed(11'd0))
            begin
              G_Out_o <= 8'd0;
            end
          else
            begin
              if ($signed(l) > $signed(11'd255))
                begin
                  G_Out_o <= 8'd255;
                end
              else
                begin
                  G_Out_o <= l[7 : 0];
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          B_Out_o <= 8'd0;
        end
      else
        begin
          if ($signed(o) < $signed(11'd0))
            begin
              B_Out_o <= 8'd0;
            end
          else
            begin
              if ($signed(o) > $signed(11'd255))
                begin
                  B_Out_o <= 8'd255;
                end
              else
                begin
                  B_Out_o <= o[7 : 0];
                end
            end
        end
    end
  ramDualPort_bayer #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) O0l(.clk(SYS_CLK_I),
                                                                                                                      .addr_a(o0),
                                                                                                                      .data_a(i0),
                                                                                                                      .we_a(l1),
                                                                                                                      .addr_b(li),
                                                                                                                      .we_b(lo),
                                                                                                                      .q_b(OIl));
  ramDualPort_bayer #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) I0l(.clk(SYS_CLK_I),
                                                                                                                      .addr_a(o0),
                                                                                                                      .data_a(i0),
                                                                                                                      .we_a(o1),
                                                                                                                      .addr_b(oi),
                                                                                                                      .we_b(oo),
                                                                                                                      .q_b(IIl));
  ramDualPort_bayer #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) l0l(.clk(SYS_CLK_I),
                                                                                                                      .addr_a(o0),
                                                                                                                      .data_a(i0),
                                                                                                                      .we_a(i1),
                                                                                                                      .addr_b(ii),
                                                                                                                      .we_b(io),
                                                                                                                      .q_b(lIl));
  ramDualPort_bayer #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) o0l(.clk(SYS_CLK_I),
                                                                                                                      .addr_a(o0),
                                                                                                                      .data_a(i0),
                                                                                                                      .we_a(Oo),
                                                                                                                      .addr_b(OOI),
                                                                                                                      .we_b(Oi),
                                                                                                                      .q_b(oIl));
  ramDualPort_bayer #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) i0l(.clk(SYS_CLK_I),
                                                                                                                      .addr_a(o0),
                                                                                                                      .data_a(i0),
                                                                                                                      .we_a(Io),
                                                                                                                      .addr_b(IOI),
                                                                                                                      .we_b(Ii),
                                                                                                                      .q_b(iIl));
  Median #(.g_DATAWIDTH((g_DATAWIDTH / 3) + 1)) lI0(.SYS_CLK_I(SYS_CLK_I),
                                                    .RESET_n_I(RESET_n_I),
                                                    .A11_i(I1l),
                                                    .A12_i(l1l),
                                                    .A13_i(o1l),
                                                    .A21_i(i1l),
                                                    .A22_i(Ool),
                                                    .A23_i(Iol),
                                                    .A31_i(lol),
                                                    .A32_i(ool),
                                                    .A33_i(iol),
                                                    .Median(OI0));
  Median #(.g_DATAWIDTH((g_DATAWIDTH / 3) + 1)) oI0(.SYS_CLK_I(SYS_CLK_I),
                                                    .RESET_n_I(RESET_n_I),
                                                    .A11_i(Oil),
                                                    .A12_i(Iil),
                                                    .A13_i(lil),
                                                    .A21_i(oil),
                                                    .A22_i(iil),
                                                    .A23_i(OO0),
                                                    .A31_i(IO0),
                                                    .A32_i(lO0),
                                                    .A33_i(oO0),
                                                    .Median(II0));
endmodule
