/*****************************************************************************************************************************
--
--    File Name    : Image_Edge_Detection.v 

--    Description  : This module implements Image Edge detection .

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

module ImageEdgeDetection #(parameter g_DATAWIDTH = 8,
                            parameter g_X_RES_WIDTH = 11,
                            parameter g_DISPLAY_RESOLUTION = 1280,
                            parameter g_VERT_DISPLAY_RESOLUTION = 720)
                          (input wire SYS_CLK_I,
                           input wire RESET_n_I,
                           input wire[(g_DATAWIDTH) - 1 : 0] Y_in_i,
                           input wire Data_In_Vld_i,
                           output reg[(g_DATAWIDTH) - 1 : 0] Y_Out_o,
                           output reg data_Vld_out_o);
  parameter[0 : 0] O = 1'b0,
                   I = 1'b1;
  parameter[2 : 0] l = 3'h0,
                   o = 3'h1,
                   i = 3'h2,
                   OI = 3'h3,
                   II = 3'h4,
                   lI = 3'h5;
  reg[g_X_RES_WIDTH - 1 : 0] oI;
  reg[g_DATAWIDTH - 1 : 0] iI;
  reg Ol;
  reg[2 : 0] Il;
  reg[g_X_RES_WIDTH - 1 : 0] ll;
  reg ol;
  reg il;
  reg O0;
  reg I0;
  reg l0;
  reg o0;
  reg i0;
  reg O1;
  reg I1;
  reg l1;
  reg o1;
  reg[g_X_RES_WIDTH - 1 : 0] i1;
  reg[g_X_RES_WIDTH - 1 : 0] Oo;
  reg[g_X_RES_WIDTH - 1 : 0] Io;
  reg[g_X_RES_WIDTH - 1 : 0] lo;
  reg[g_X_RES_WIDTH - 1 : 0] oo;
  reg[g_X_RES_WIDTH - 1 : 0] io;
  reg[g_X_RES_WIDTH - 1 : 0] Oi;
  reg[g_X_RES_WIDTH - 1 : 0] Ii;
  reg[g_X_RES_WIDTH - 1 : 0] li;
  reg[g_X_RES_WIDTH - 1 : 0] oi;
  reg[g_DATAWIDTH - 1 : 0] ii;
  reg[g_DATAWIDTH - 1 : 0] OOI;
  reg[g_DATAWIDTH - 1 : 0] IOI;
  reg[g_DATAWIDTH - 1 : 0] lOI;
  reg[g_DATAWIDTH - 1 : 0] oOI;
  reg[g_DATAWIDTH - 1 : 0] iOI;
  reg[g_DATAWIDTH - 1 : 0] OII;
  reg[g_DATAWIDTH - 1 : 0] III;
  reg[g_DATAWIDTH - 1 : 0] lII;
  reg[g_DATAWIDTH - 1 : 0] oII;
  reg[g_DATAWIDTH - 1 : 0] iII;
  reg[g_DATAWIDTH - 1 : 0] OlI;
  reg[(g_DATAWIDTH) - 1 : 0] IlI;
  reg[(g_DATAWIDTH) - 1 : 0] llI;
  reg[(g_DATAWIDTH) - 1 : 0] olI;
  reg[(g_DATAWIDTH) - 1 : 0] ilI;
  reg[(g_DATAWIDTH) - 1 : 0] O0I;
  reg[(g_DATAWIDTH) - 1 : 0] I0I;
  reg[(g_DATAWIDTH) - 1 : 0] l0I;
  reg[(g_DATAWIDTH) - 1 : 0] o0I;
  reg[g_X_RES_WIDTH - 1 : 0] i0I;
  reg[g_X_RES_WIDTH - 1 : 0] O1I;
  reg[g_DATAWIDTH - 1 : 0] I1I;
  reg[g_DATAWIDTH - 1 : 0] l1I;
  reg[g_DATAWIDTH - 1 : 0] o1I;
  reg[g_DATAWIDTH - 1 : 0] i1I;
  reg[g_DATAWIDTH - 1 : 0] OoI;
  reg[2 : 0] IoI;
  reg[2 : 0] loI;
  reg ooI;
  reg[g_X_RES_WIDTH - 1 : 0] ioI;
  reg[0 : 0] OiI;
  reg IiI;
  reg[2 : 0] liI;
  reg oiI;
  reg iiI;
  reg OOl;
  wire[g_DATAWIDTH - 1 : 0] IOl;
  wire[g_DATAWIDTH - 1 : 0] lOl;
  wire[g_DATAWIDTH - 1 : 0] oOl;
  wire[g_DATAWIDTH - 1 : 0] iOl;
  wire[g_DATAWIDTH - 1 : 0] OIl;
  wire[(g_DATAWIDTH) - 1 : 0] IIl;
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          oI <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if (oI == g_DISPLAY_RESOLUTION - 1)
            begin
              oI <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (Ol == 1'b1)
                begin
                  oI <= oI + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Il <= 3'b000;
        end
      else
        begin
          if ((Il == 3'b100 && oI == g_DISPLAY_RESOLUTION - 1) || (ll == g_VERT_DISPLAY_RESOLUTION - 1 && oI == g_DISPLAY_RESOLUTION - 1))
            begin
              Il <= 3'b000;
            end
          else
            begin
              if (oI == g_DISPLAY_RESOLUTION - 1)
                begin
                  Il <= Il + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          ll <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if (ll == g_VERT_DISPLAY_RESOLUTION)
            begin
              ll <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (oI == g_DISPLAY_RESOLUTION - 1 && Ol == 1'b1)
                begin
                  ll <= ll + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          iiI <= 1'b0;
        end
      else
        begin
          if (ll == g_VERT_DISPLAY_RESOLUTION)
            begin
              iiI <= 1'b1;
            end
          else
            if (OOl)
              begin
                iiI <= 1'b0;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          iI <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          iI <= { Y_in_i };
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Ol <= 1'b0;
        end
      else
        begin
          Ol <= Data_In_Vld_i;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          il <= 1'b0;
          O0 <= 1'b0;
          I0 <= 1'b0;
          l0 <= 1'b0;
          o0 <= 1'b0;
        end
      else
        if (Data_In_Vld_i)
          begin
            case (Il)
              3'd0:
                begin
                  il <= 1'b1;
                  O0 <= 1'b0;
                  I0 <= 1'b0;
                  l0 <= 1'b0;
                  o0 <= 1'b0;
                end
              3'd1:
                begin
                  il <= 1'b0;
                  O0 <= 1'b1;
                  I0 <= 1'b0;
                  l0 <= 1'b0;
                  o0 <= 1'b0;
                end
              3'd2:
                begin
                  il <= 1'b0;
                  O0 <= 1'b0;
                  I0 <= 1'b1;
                  l0 <= 1'b0;
                  o0 <= 1'b0;
                end
              3'd3:
                begin
                  il <= 1'b0;
                  O0 <= 1'b0;
                  I0 <= 1'b0;
                  l0 <= 1'b1;
                  o0 <= 1'b0;
                end
              3'd4:
                begin
                  il <= 1'b0;
                  O0 <= 1'b0;
                  I0 <= 1'b0;
                  l0 <= 1'b0;
                  o0 <= 1'b1;
                end
              default:
                begin
                  il <= 1'b0;
                  O0 <= 1'b0;
                  I0 <= 1'b0;
                  l0 <= 1'b0;
                  o0 <= 1'b0;
                end
            endcase
          end
        else
          begin
            il <= 1'b0;
            O0 <= 1'b0;
            I0 <= 1'b0;
            l0 <= 1'b0;
            o0 <= 1'b0;
          end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          ol <= 1'b0;
        end
      else
        begin
          if ((Il >= 3'b010) && (oI >= 11'h004))
            begin
              ol <= 1'b1;
            end
          else
            if ((O1I == g_VERT_DISPLAY_RESOLUTION - 1) && (i0I == g_DISPLAY_RESOLUTION - 1))
              begin
                ol <= 1'b0;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          liI <= l;
        end
      else
        begin
          case (liI)
            l:
              begin
                if (Il >= 3'b010 && oI >= 11'h004 || (ll == 0))
                  begin
                    liI <= o;
                  end
                else
                  begin
                    liI <= l;
                  end
              end
            o:
              begin
                if (i0I == g_DISPLAY_RESOLUTION)
                  begin
                    liI <= lI;
                  end
                else
                  begin
                    liI <= o;
                  end
              end
            lI:
              begin
                if (iiI)
                  begin
                    liI <= OI;
                  end
                else
                  begin
                    liI <= i;
                  end
              end
            i:
              begin
                if (Data_In_Vld_i && oI >= 11'h004)
                  begin
                    liI <= OI;
                  end
                else
                  begin
                    liI <= i;
                  end
              end
            OI:
              begin
                if (O1I == g_VERT_DISPLAY_RESOLUTION - 1)
                  begin
                    liI <= II;
                  end
                else
                  begin
                    liI <= o;
                  end
              end
            II:
              begin
                if (i0I == g_DISPLAY_RESOLUTION - 1)
                  begin
                    liI <= l;
                  end
                else
                  begin
                    liI <= II;
                  end
              end
            default:
              liI <= l;
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          oiI <= 1'b0;
          OOl <= 1'b0;
        end
      else
        begin
          case (liI)
            l:
              begin
                oiI <= 1'b0;
                OOl <= 1'b0;
              end
            o:
              begin
                if (i0I == g_DISPLAY_RESOLUTION)
                  begin
                    oiI <= 1'b1;
                  end
                else
                  begin
                    oiI <= 1'b0;
                  end
              end
            lI:
              begin
                oiI <= 1'b1;
              end
            i:
              begin
                if (Data_In_Vld_i && oI >= 11'h004 || (ll == 0))
                  begin
                    oiI <= 1'b0;
                  end
                else
                  begin
                    oiI <= oiI;
                  end
              end
            OI:
              begin
                oiI <= 1'b0;
              end
            II:
              begin
                if (i0I == g_DISPLAY_RESOLUTION - 1)
                  begin
                    oiI <= 1'b0;
                    OOl <= 1'b1;
                  end
                else
                  begin
                    oiI <= oiI;
                    OOl <= 1'b0;
                  end
              end
            default:
              begin
                oiI <= 1'b0;
                OOl <= 1'b0;
              end
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          i0I <= { g_X_RES_WIDTH { 1'b1 } };
        end
      else
        begin
          if (ol == 1'b0 || i0I > g_DISPLAY_RESOLUTION - 1)
            begin
              i0I <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            if (ol == 1'b1)
              begin
                if (oiI)
                  i0I <= { g_X_RES_WIDTH { 1'b0 } };
                else
                  i0I <= i0I + 1'b1;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          O1I <= { g_X_RES_WIDTH { 1'b1 } };
        end
      else
        begin
          if (ol == 1'b0)
            begin
              O1I <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (i0I > g_DISPLAY_RESOLUTION - 1 && ol == 1'b1)
                begin
                  O1I <= O1I + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          IoI <= 3'd0;
        end
      else
        begin
          if (ol == 1'b0 || (IoI == 3'd4 && i0I == g_DISPLAY_RESOLUTION))
            begin
              IoI <= 3'd0;
            end
          else
            begin
              if (i0I >= g_DISPLAY_RESOLUTION && ol == 1'b1)
                begin
                  IoI <= IoI + 12'd1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          loI <= 3'd0;
        end
      else
        begin
          loI <= IoI;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (RESET_n_I == 1'b0)
        begin
          i0 <= 1'b1;
          O1 <= 1'b1;
          I1 <= 1'b1;
          l1 <= 1'b1;
          o1 <= 1'b1;
        end
      else
        begin
          case (IoI)
            3'd0:
              begin
                i0 <= 1'b0;
                O1 <= 1'b0;
                I1 <= 1'b0;
                l1 <= 1'b1;
                o1 <= 1'b1;
              end
            3'd1:
              begin
                i0 <= 1'b1;
                O1 <= 1'b0;
                I1 <= 1'b0;
                l1 <= 1'b0;
                o1 <= 1'b1;
              end
            3'd2:
              begin
                i0 <= 1'b1;
                O1 <= 1'b1;
                I1 <= 1'b0;
                l1 <= 1'b0;
                o1 <= 1'b0;
              end
            3'd3:
              begin
                i0 <= 1'b0;
                O1 <= 1'b1;
                I1 <= 1'b1;
                l1 <= 1'b0;
                o1 <= 1'b0;
              end
            3'd4:
              begin
                i0 <= 1'b0;
                O1 <= 1'b0;
                I1 <= 1'b1;
                l1 <= 1'b1;
                o1 <= 1'b0;
              end
            default:
              ;
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          i1 <= { g_X_RES_WIDTH { 1'b0 } };
          Oo <= { g_X_RES_WIDTH { 1'b0 } };
          Io <= { g_X_RES_WIDTH { 1'b0 } };
          lo <= { g_X_RES_WIDTH { 1'b0 } };
          oo <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if ((ol == 1'b1 && (i1 > g_DISPLAY_RESOLUTION - 1 || Oo > g_DISPLAY_RESOLUTION - 1 || Io > g_DISPLAY_RESOLUTION - 1 || lo > g_DISPLAY_RESOLUTION - 1 || oo > g_DISPLAY_RESOLUTION - 1)))
            begin
              i1 <= { g_X_RES_WIDTH { 1'b0 } };
              Oo <= { g_X_RES_WIDTH { 1'b0 } };
              Io <= { g_X_RES_WIDTH { 1'b0 } };
              lo <= { g_X_RES_WIDTH { 1'b0 } };
              oo <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (ol == 1'b1)
                begin
                  if (oiI == 1'b1)
                    begin
                      i1 <= i1;
                      Oo <= Oo;
                      Io <= Io;
                      lo <= lo;
                      oo <= oo;
                    end
                  else
                    begin
                      case (loI)
                        3'd0:
                          begin
                            i1 <= i1 + 1'b1;
                            Oo <= Oo + 1'b1;
                            Io <= Io + 1'b1;
                          end
                        3'd1:
                          begin
                            Oo <= Oo + 1'b1;
                            Io <= Io + 1'b1;
                            lo <= lo + 1'b1;
                          end
                        3'd2:
                          begin
                            Io <= Io + 1'b1;
                            lo <= lo + 1'b1;
                            oo <= oo + 1'b1;
                          end
                        3'd3:
                          begin
                            lo <= lo + 1'b1;
                            oo <= oo + 1'b1;
                            i1 <= i1 + 1'b1;
                          end
                        3'd4:
                          begin
                            oo <= oo + 1'b1;
                            i1 <= i1 + 1'b1;
                            Oo <= Oo + 1'b1;
                          end
                        default:
                          begin
                            i1 <= i1;
                            Oo <= Oo;
                            Io <= Io;
                            lo <= lo;
                            oo <= oo;
                          end
                      endcase
                    end
                end
              else
                if (ol == 1'b0)
                  begin
                    i1 <= { g_X_RES_WIDTH { 1'b0 } };
                    Oo <= { g_X_RES_WIDTH { 1'b0 } };
                    Io <= { g_X_RES_WIDTH { 1'b0 } };
                    lo <= { g_X_RES_WIDTH { 1'b0 } };
                    oo <= { g_X_RES_WIDTH { 1'b0 } };
                  end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          io <= { g_X_RES_WIDTH { 1'b0 } };
          Oi <= { g_X_RES_WIDTH { 1'b0 } };
          Ii <= { g_X_RES_WIDTH { 1'b0 } };
          li <= { g_X_RES_WIDTH { 1'b0 } };
          oi <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          io <= i1;
          Oi <= Oo;
          Ii <= Io;
          li <= lo;
          oi <= oo;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          I1I <= { g_DATAWIDTH { 1'b0 } };
          l1I <= { g_DATAWIDTH { 1'b0 } };
          o1I <= { g_DATAWIDTH { 1'b0 } };
          i1I <= { g_DATAWIDTH { 1'b0 } };
          OoI <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          I1I <= IOl;
          l1I <= lOl;
          o1I <= oOl;
          i1I <= iOl;
          OoI <= OIl;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          ii <= { g_DATAWIDTH { 1'b0 } };
          oOI <= { g_DATAWIDTH { 1'b0 } };
          lII <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (Ii == 11'd1 || li == 11'd1 || oi == 11'd1)
            begin
              case (loI)
                3'd0:
                  begin
                    ii <= I1I;
                    oOI <= l1I;
                    lII <= o1I;
                  end
                3'd1:
                  begin
                    ii <= l1I;
                    oOI <= o1I;
                    lII <= i1I;
                  end
                3'd2:
                  begin
                    ii <= o1I;
                    oOI <= i1I;
                    lII <= OoI;
                  end
                3'd3:
                  begin
                    ii <= i1I;
                    oOI <= OoI;
                    lII <= I1I;
                  end
                3'd4:
                  begin
                    ii <= OoI;
                    oOI <= I1I;
                    lII <= l1I;
                  end
                default:
                  begin
                    ii <= ii;
                    oOI <= oOI;
                    lII <= lII;
                  end
              endcase
            end
          else
            begin
              if (Ii >= 11'd5 || li >= 11'd5 || oi >= 11'd5)
                begin
                  ii <= OOI;
                  oOI <= iOI;
                  lII <= oII;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          OOI <= { g_DATAWIDTH { 1'b0 } };
          iOI <= { g_DATAWIDTH { 1'b0 } };
          oII <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (Ii == 11'd2 || li == 11'd2 || oi == 11'd2)
            begin
              case (loI)
                3'd0:
                  begin
                    OOI <= I1I;
                    iOI <= l1I;
                    oII <= o1I;
                  end
                3'd1:
                  begin
                    OOI <= l1I;
                    iOI <= o1I;
                    oII <= i1I;
                  end
                3'd2:
                  begin
                    OOI <= o1I;
                    iOI <= i1I;
                    oII <= OoI;
                  end
                3'd3:
                  begin
                    OOI <= i1I;
                    iOI <= OoI;
                    oII <= I1I;
                  end
                3'd4:
                  begin
                    OOI <= OoI;
                    iOI <= I1I;
                    oII <= l1I;
                  end
                default:
                  begin
                    OOI <= OOI;
                    iOI <= iOI;
                    oII <= oII;
                  end
              endcase
            end
          else
            begin
              if (Ii >= 11'd5 || li >= 11'd5 || oi >= 11'd5)
                begin
                  OOI <= IOI;
                  iOI <= OII;
                  oII <= iII;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          IOI <= { g_DATAWIDTH { 1'b0 } };
          OII <= { g_DATAWIDTH { 1'b0 } };
          iII <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (Ii == 11'd3 || li == 11'd3 || oi == 11'd3)
            begin
              case (loI)
                3'd0:
                  begin
                    IOI <= I1I;
                    OII <= l1I;
                    iII <= o1I;
                  end
                3'd1:
                  begin
                    IOI <= l1I;
                    OII <= o1I;
                    iII <= i1I;
                  end
                3'd2:
                  begin
                    IOI <= o1I;
                    OII <= i1I;
                    iII <= OoI;
                  end
                3'd3:
                  begin
                    IOI <= i1I;
                    OII <= OoI;
                    iII <= I1I;
                  end
                3'd4:
                  begin
                    IOI <= OoI;
                    OII <= I1I;
                    iII <= l1I;
                  end
                default:
                  begin
                    IOI <= IOI;
                    OII <= OII;
                    iII <= iII;
                  end
              endcase
            end
          else
            begin
              if (Ii >= 11'd5 || li >= 11'd5 || oi >= 11'd5)
                begin
                  IOI <= lOI;
                  OII <= III;
                  iII <= OlI;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          lOI <= { g_DATAWIDTH { 1'b0 } };
          III <= { g_DATAWIDTH { 1'b0 } };
          OlI <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (Ii >= 11'd4 || li >= 11'd4 || oi >= 11'd4)
            begin
              case (loI)
                3'd0:
                  begin
                    lOI <= I1I;
                    III <= l1I;
                    OlI <= o1I;
                  end
                3'd1:
                  begin
                    lOI <= l1I;
                    III <= o1I;
                    OlI <= i1I;
                  end
                3'd2:
                  begin
                    lOI <= o1I;
                    III <= i1I;
                    OlI <= OoI;
                  end
                3'd3:
                  begin
                    lOI <= i1I;
                    III <= OoI;
                    OlI <= I1I;
                  end
                3'd4:
                  begin
                    lOI <= OoI;
                    III <= I1I;
                    OlI <= l1I;
                  end
                default:
                  begin
                    lOI <= lOI;
                    III <= III;
                    OlI <= OlI;
                  end
              endcase
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          ooI <= 1'b0;
        end
      else
        begin
          if (io == g_DISPLAY_RESOLUTION || Oi == g_DISPLAY_RESOLUTION || Ii == g_DISPLAY_RESOLUTION || li == g_DISPLAY_RESOLUTION || oi == g_DISPLAY_RESOLUTION)
            begin
              ooI <= 1'b1;
            end
          else
            if (io <= 11'd4 || Oi >= 11'd4 || Ii >= 11'd4 || li >= 11'd4 || oi >= 11'd4)
              begin
                ooI <= 1'b0;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          IlI <= { (g_DATAWIDTH) { 1'b0 } };
          llI <= { (g_DATAWIDTH) { 1'b0 } };
          olI <= { (g_DATAWIDTH) { 1'b0 } };
          ilI <= { (g_DATAWIDTH) { 1'b0 } };
          O0I <= { (g_DATAWIDTH) { 1'b0 } };
          I0I <= { (g_DATAWIDTH) { 1'b0 } };
          l0I <= { (g_DATAWIDTH) { 1'b0 } };
          o0I <= { (g_DATAWIDTH) { 1'b0 } };
        end
      else
        begin
          if (Ii >= 11'd5 || li >= 11'd5 || oi >= 11'd5 || Oi >= 11'd5 || io >= 11'd5 || ooI)
            begin
              IlI <= ii;
              llI <= OOI;
              olI <= IOI;
              ilI <= oOI;
              O0I <= OII;
              I0I <= lII;
              l0I <= oII;
              o0I <= iII;
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          OiI <= O;
          ioI <= 11'd0;
          IiI <= 1'b0;
        end
      else
        begin
          case (OiI)
            O:
              begin
                if (i0I == 11'd10)
                  begin
                    ioI <= ioI + 1'b1;
                    IiI <= 1'b1;
                    OiI <= I;
                  end
                else
                  begin
                    OiI <= O;
                    ioI <= 11'd0;
                    IiI <= 1'b0;
                  end
              end
            I:
              begin
                if (ioI == g_DISPLAY_RESOLUTION)
                  begin
                    OiI <= O;
                    ioI <= 11'd0;
                    IiI <= 1'b0;
                  end
                else
                  begin
                    ioI <= ioI + 1'b1;
                    IiI <= 1'b1;
                    OiI <= I;
                  end
              end
            default:
              begin
                OiI <= O;
              end
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Y_Out_o <= 8'd0;
          data_Vld_out_o <= 1'b0;
        end
      else
        begin
          Y_Out_o <= IIl;
          data_Vld_out_o <= IiI;
        end
    end
  ramDualPort_edge #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) lIl(.clk(SYS_CLK_I),
                                                                                                                     .addr_a(oI),
                                                                                                                     .data_a(iI),
                                                                                                                     .we_a(il),
                                                                                                                     .addr_b(i1),
                                                                                                                     .we_b(i0),
                                                                                                                     .q_b(IOl));
  ramDualPort_edge #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) oIl(.clk(SYS_CLK_I),
                                                                                                                     .addr_a(oI),
                                                                                                                     .data_a(iI),
                                                                                                                     .we_a(O0),
                                                                                                                     .addr_b(Oo),
                                                                                                                     .we_b(O1),
                                                                                                                     .q_b(lOl));
  ramDualPort_edge #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) iIl(.clk(SYS_CLK_I),
                                                                                                                     .addr_a(oI),
                                                                                                                     .data_a(iI),
                                                                                                                     .we_a(I0),
                                                                                                                     .addr_b(Io),
                                                                                                                     .we_b(I1),
                                                                                                                     .q_b(oOl));
  ramDualPort_edge #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) Oll(.clk(SYS_CLK_I),
                                                                                                                     .addr_a(oI),
                                                                                                                     .data_a(iI),
                                                                                                                     .we_a(l0),
                                                                                                                     .addr_b(lo),
                                                                                                                     .we_b(l1),
                                                                                                                     .q_b(iOl));
  ramDualPort_edge #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) Ill(.clk(SYS_CLK_I),
                                                                                                                     .addr_a(oI),
                                                                                                                     .data_a(iI),
                                                                                                                     .we_a(o0),
                                                                                                                     .addr_b(oo),
                                                                                                                     .we_b(o1),
                                                                                                                     .q_b(OIl));
  sobel #(.g_DATA_BITWIDTH(g_DATAWIDTH)) lll(.SYS_CLK_I(SYS_CLK_I),
                                             .nRESET_I(RESET_n_I),
                                             .p0_i(IlI),
                                             .p1_i(llI),
                                             .p2_i(olI),
                                             .p3_i(ilI),
                                             .p5_i(O0I),
                                             .p6_i(I0I),
                                             .p7_i(l0I),
                                             .p8_i(o0I),
                                             .DATA_OUT_o(IIl));
endmodule
