/*****************************************************************************************************************************
--
--    File Name    : YCbCr2RGB.v 

--    Description  : This module implements YCbCr to RGB color conversion
--                   

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

`timescale 1ns/1ps
module YCbCr2RGB #(parameter g_RGB_DATA_BIT_WIDTH = 8,
                   parameter g_YCbCr_DATA_BIT_WIDTH = 8)
                 (input wire CLOCK_I,
                  input wire RESET_N_I,
                  input wire[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Y_i,
                  input wire[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Cb_i,
                  input wire[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Cr_i,
                  input wire data_enable_i,
                  input wire horz_sync_i,
                  input wire vert_sync_i,
                  input wire hactive_i,
                  input wire vactive_i,
                  output reg[(g_RGB_DATA_BIT_WIDTH - 1) : 0] Red_out_o,
                  output reg[(g_RGB_DATA_BIT_WIDTH - 1) : 0] Green_out_o,
                  output reg[(g_RGB_DATA_BIT_WIDTH - 1) : 0] Blue_out_o,
                  output reg data_enable_o,
                  output reg horz_sync_o,
                  output reg vert_sync_o,
                  output reg hactive_o,
                  output reg vactive_o);
  parameter O = 15;
  parameter I = 32768;
  parameter l = 44;
  parameter o = 38142;
  parameter i = 52298;
  parameter OI = 7304380;
  parameter II = 38142;
  parameter lI = 26640;
  parameter oI = 12812;
  parameter iI = 4439671;
  parameter Ol = 38142;
  parameter Il = 66126;
  parameter ll = 9074377;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] ol;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] il;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] O0;
  reg[(l - 1) : 0] I0;
  reg[(l - 1) : 0] l0;
  reg[(l - 1) : 0] o0;
  reg[(l - 1) : 0] i0;
  reg[(l - 1) : 0] O1;
  reg[(l - 1) : 0] I1;
  reg[(l - 1) : 0] l1;
  reg[(l - 1) : 0] o1;
  reg[(l - 1) : 0] i1;
  reg[(l - 1) : 0] Oo;
  reg[(l - 1) : 0] Io;
  reg[(l - 1) : 0] lo;
  reg[(l - 1) : 0] oo;
  reg[(l - 1) : 0] io;
  reg signed[(l - 1) : 0] Oi;
  reg signed[(l - 1) : 0] Ii;
  reg signed[(l - 1) : 0] li;
  reg signed[(l - O - 1) : 0] oi;
  reg signed[(l - O - 1) : 0] ii;
  reg signed[(l - O - 1) : 0] OOI;
  reg IOI;
  reg lOI;
  reg oOI;
  reg iOI;
  reg OII;
  reg III;
  reg lII;
  reg oII;
  reg iII;
  reg OlI;
  reg IlI;
  reg llI;
  reg olI;
  reg ilI;
  reg O0I;
  reg I0I;
  reg l0I;
  reg o0I;
  reg i0I;
  reg O1I;
  reg I1I;
  reg l1I;
  reg o1I;
  reg i1I;
  reg OoI;
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          ol <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          il <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          O0 <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          IOI <= 1'b0;
          III <= 1'b0;
          IlI <= 1'b0;
          I0I <= 1'b0;
          I1I <= 1'b0;
        end
      else
        begin
          ol <= Y_i;
          il <= Cb_i;
          O0 <= Cr_i;
          IOI <= horz_sync_i;
          III <= vert_sync_i;
          IlI <= data_enable_i;
          I0I <= hactive_i;
          I1I <= vactive_i;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          I0 <= { l { 1'b0 } };
          l0 <= { l { 1'b0 } };
        end
      else
        begin
          I0 <= o * ol;
          l0 <= i * O0;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          o0 <= { l { 1'b0 } };
          i0 <= { l { 1'b0 } };
          O1 <= { l { 1'b0 } };
        end
      else
        begin
          o0 <= II * ol;
          i0 <= lI * O0;
          O1 <= oI * il;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          I1 <= { l { 1'b0 } };
          l1 <= { l { 1'b0 } };
        end
      else
        begin
          I1 <= Ol * ol;
          l1 <= Il * il;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          lOI <= 1'b0;
          lII <= 1'b0;
          llI <= 1'b0;
          l0I <= 1'b0;
          l1I <= 1'b0;
        end
      else
        begin
          lOI <= IOI;
          lII <= III;
          llI <= IlI;
          l0I <= I0I;
          l1I <= I1I;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          o1 <= { l { 1'b0 } };
          i1 <= { l { 1'b0 } };
          Oo <= { l { 1'b0 } };
          Io <= { l { 1'b0 } };
          lo <= { l { 1'b0 } };
          oo <= { l { 1'b0 } };
          io <= { l { 1'b0 } };
          oOI <= 1'b0;
          oII <= 1'b0;
          olI <= 1'b0;
          o0I <= 1'b0;
          o1I <= 1'b0;
        end
      else
        begin
          o1 <= I0;
          i1 <= l0;
          Oo <= o0;
          Io <= i0;
          lo <= O1;
          oo <= I1;
          io <= l1;
          oOI <= lOI;
          oII <= lII;
          olI <= llI;
          o0I <= l0I;
          o1I <= l1I;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          Oi <= { l { 1'b0 } };
          Ii <= { l { 1'b0 } };
          li <= { l { 1'b0 } };
          iOI <= 1'b0;
          iII <= 1'b0;
          ilI <= 1'b0;
          i0I <= 1'b0;
          i1I <= 1'b0;
        end
      else
        begin
          Oi <= $signed({ 1'b0, o1 }) + $signed({ 1'b0, i1 }) - $signed({ 1'b0, OI });
          Ii <= $signed({ 1'b0, Oo }) - $signed({ 1'b0, Io }) - $signed({ 1'b0, lo }) + $signed({ 1'b0, iI });
          li <= $signed({ 1'b0, oo }) + $signed({ 1'b0, io }) - $signed({ 1'b0, ll });
          iOI <= oOI;
          iII <= oII;
          ilI <= olI;
          i0I <= o0I;
          i1I <= o1I;
        end
    end
  always
    @(posedge CLOCK_I or
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          oi <= { (l - O) { 1'b0 } };
          ii <= { (l - O) { 1'b0 } };
          OOI <= { (l - O) { 1'b0 } };
          OII <= 1'b0;
          OlI <= 1'b0;
          O0I <= 1'b0;
          O1I <= 1'b0;
          OoI <= 1'b0;
        end
      else
        begin
          oi <= Oi >> O;
          ii <= Ii >> O;
          OOI <= li >> O;
          OII <= iOI;
          OlI <= iII;
          O0I <= ilI;
          O1I <= i0I;
          OoI <= i1I;
        end
    end
  always
    @(posedge CLOCK_I or
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          Red_out_o <= { g_RGB_DATA_BIT_WIDTH { 1'b0 } };
        end
      else
        begin
          if (oi > $signed({ { (l - O - 8) { 1'b0 } }, 8'd255 }))
            Red_out_o <= 8'd255;
          else
            if (oi < $signed({ (l - O) { 1'b0 } }))
              Red_out_o <= { g_RGB_DATA_BIT_WIDTH { 1'b0 } };
            else
              Red_out_o <= oi[7 : 0];
        end
    end
  always
    @(posedge CLOCK_I or
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          Green_out_o <= { g_RGB_DATA_BIT_WIDTH { 1'b0 } };
        end
      else
        begin
          if (ii > $signed({ { (l - O - 8) { 1'b0 } }, 8'd255 }))
            Green_out_o <= 8'd255;
          else
            if (ii < $signed({ (l - O) { 1'b0 } }))
              Green_out_o <= { g_RGB_DATA_BIT_WIDTH { 1'b0 } };
            else
              Green_out_o <= ii[7 : 0];
        end
    end
  always
    @(posedge CLOCK_I or
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          Blue_out_o <= { g_RGB_DATA_BIT_WIDTH { 1'b0 } };
        end
      else
        begin
          if (OOI > $signed({ { (l - O - 8) { 1'b0 } }, 8'd255 }))
            Blue_out_o <= 8'd255;
          else
            if (OOI < $signed({ (l - O) { 1'b0 } }))
              Blue_out_o <= { g_RGB_DATA_BIT_WIDTH { 1'b0 } };
            else
              Blue_out_o <= OOI[7 : 0];
        end
    end
  always
    @(posedge CLOCK_I or
      negedge RESET_N_I)
    begin
      if (~RESET_N_I)
        begin
          horz_sync_o <= 1'b0;
          vert_sync_o <= 1'b0;
          data_enable_o <= 1'b0;
          hactive_o <= 1'b0;
          vactive_o <= 1'b0;
        end
      else
        begin
          horz_sync_o <= OII;
          vert_sync_o <= OlI;
          data_enable_o <= O0I;
          hactive_o <= O1I;
          vactive_o <= OoI;
        end
    end
endmodule
