/*****************************************************************************************************************************
--
--    File Name    : wr_master_req_latch.v

--    Description  : This module latches the external master write request to be given as input to AXI_arbiter module
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2013 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 
--******************************************************************************************************************************/
`timescale 1ns/1ps
module wr_master_req_latch(input wire clock,
                           input wire nreset,
                           input wire wr_req_1,
                           input wire wr_done_1,
                           output reg wr_req_arb_1,
                           input wire wr_req_2,
                           input wire wr_done_2,
                           output reg wr_req_arb_2);
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          wr_req_arb_1 <= 1'b0;
        end
      else
        begin
          if (wr_done_1 && !wr_req_1)
            begin
              wr_req_arb_1 <= 1'b0;
            end
          else
            if (wr_req_1)
              begin
                wr_req_arb_1 <= 1'b1;
              end
            else
              begin
                wr_req_arb_1 <= wr_req_arb_1;
              end
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          wr_req_arb_2 <= 1'b0;
        end
      else
        begin
          if (wr_done_2 && !wr_req_2)
            begin
              wr_req_arb_2 <= 1'b0;
            end
          else
            if (wr_req_2)
              begin
                wr_req_arb_2 <= 1'b1;
              end
            else
              begin
                wr_req_arb_2 <= wr_req_arb_2;
              end
        end
    end
endmodule
