/*****************************************************************************************************************************
--
--    File Name    : ddr_read_controller.v

--    Description  : This module setups up the DDR data read for the display controller
--                   

-- Targeted device : Microsemi-SoC
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

`timescale 1ns/1ps
module ddr_read_controller #(parameter g_DDR_AXI_AWIDTH = 32,
                             parameter g_INPUT_X_W_RES_WIDTH = 12,
                             parameter g_INPUT_Y_H_RES_WIDTH = 12,
                             parameter g_VIDEO_PIXEL_FROM_DDR_DEPTH = 3,
                             parameter g_INITIAL_LINES_TO_BUFFER = 2,
                             parameter g_SUBSEQUENT_LINES_TO_BUFFER = 1)
                           (input wire RESET_N_I,
                            input wire SYS_CLOCK_I,
                            input wire display_control_enable_i,
                            input wire[(g_DDR_AXI_AWIDTH - 1) : 0] frame_buff_addr_i,
                            input wire frame_ddr_addr_valid_i,
                            input wire[31 : 0] video_resolution_i,
                            output wire[(g_DDR_AXI_AWIDTH - 1) : 0] frame_buff_und_disp_o,
                            input wire start_ddr_read,
                            output wire read_req_o,
                            input wire read_ack_i,
                            input wire read_done_i,
                            output wire[(g_DDR_AXI_AWIDTH - 1) : 0] start_read_addr_o,
                            output wire[15 : 0] bytes_to_read_o,
                            input wire fifo_half_empty,
                            input wire frame_complete);
  reg ii,
      OOI,
      IOI;
  wire lOI;
  reg oOI,
      iOI,
      OII;
  wire III;
  reg lII,
      oII,
      iII;
  reg OlI;
  reg IlI;
  reg llI;
  reg olI;
  reg[15 : 0] ilI;
  reg[(g_DDR_AXI_AWIDTH - 1) : 0] O0I;
  reg[(g_DDR_AXI_AWIDTH - 1) : 0] I0I;
  reg[(g_DDR_AXI_AWIDTH - 1) : 0] l0I;
  reg[(g_INPUT_X_W_RES_WIDTH - 1) : 0] o0I;
  reg[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] i0I;
  reg[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] O1I;
  reg I1I;
  reg l1I;
  parameter o1I = g_VIDEO_PIXEL_FROM_DDR_DEPTH;
  parameter[3 : 0] i1I = 4'd0,
                   OoI = 4'd1,
                   IoI = 4'd2,
                   loI = 4'd3,
                   ooI = 4'd4,
                   ioI = 4'd5,
                   OiI = 4'd6,
                   IiI = 4'd7,
                   liI = 4'd8,
                   oiI = 4'd9,
                   iiI = 4'd10;
  reg[3 : 0] OOl;
  always
    @(posedge SYS_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          llI <= 1'b0;
          olI <= 1'b0;
        end
      else
        begin
          llI <= fifo_half_empty;
          olI <= llI;
        end
    end
  always
    @(posedge SYS_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          OlI <= 1'b0;
          IlI <= 1'b0;
        end
      else
        begin
          OlI <= start_ddr_read;
          IlI <= OlI;
        end
    end
  always
    @(posedge SYS_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          ii <= 1'b0;
          OOI <= 1'b0;
          IOI <= 1'b0;
        end
      else
        begin
          ii <= display_control_enable_i;
          OOI <= ii;
          IOI <= OOI;
        end
    end
  assign lOI = OOI & ~IOI;
  always
    @(posedge SYS_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          oOI <= 1'b0;
          iOI <= 1'b0;
          OII <= 1'b0;
        end
      else
        begin
          oOI <= frame_complete;
          iOI <= oOI;
          OII <= iOI;
        end
    end
  assign III = iOI & ~OII;
  always
    @(posedge SYS_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          lII <= 1'b0;
          oII <= 1'b0;
          iII <= 1'b0;
        end
      else
        begin
          lII <= frame_ddr_addr_valid_i;
          oII <= lII;
          iII <= oII;
        end
    end
  always
    @(posedge SYS_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          I0I <= { g_DDR_AXI_AWIDTH { 1'b0 } };
        end
      else
        begin
          if (iII)
            I0I <= frame_buff_addr_i;
          else
            I0I <= I0I;
        end
    end
  assign frame_buff_und_disp_o = O0I;
  assign read_req_o = l1I;
  assign start_read_addr_o = l0I;
  assign bytes_to_read_o = ilI;
  always
    @(posedge SYS_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          OOl <= i1I;
        end
      else
        begin
          case (OOl)
            i1I:
              begin
                if (lOI)
                  OOl <= OoI;
                else
                  OOl <= i1I;
              end
            OoI:
              begin
                OOl <= IoI;
              end
            IoI:
              begin
                OOl <= loI;
              end
            loI:
              begin
                if (IlI)
                  OOl <= ooI;
                else
                  OOl <= loI;
              end
            ooI:
              begin
                OOl <= ioI;
              end
            ioI:
              begin
                if (read_ack_i)
                  OOl <= OiI;
                else
                  OOl <= ioI;
              end
            OiI:
              begin
                if (read_done_i)
                  OOl <= IiI;
                else
                  OOl <= OiI;
              end
            IiI:
              begin
                if (olI)
                  OOl <= liI;
                else
                  OOl <= IiI;
              end
            liI:
              begin
                if (O1I == { g_INPUT_Y_H_RES_WIDTH { 1'b0 } })
                  OOl <= iiI;
                else
                  OOl <= oiI;
              end
            oiI:
              begin
                OOl <= ioI;
              end
            iiI:
              begin
                if (III)
                  begin
                    OOl <= OoI;
                  end
                else
                  begin
                    OOl <= iiI;
                  end
              end
            default:
              OOl <= i1I;
          endcase
        end
    end
  always
    @(posedge SYS_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          o0I <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
          i0I <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
          l0I <= { g_DDR_AXI_AWIDTH { 1'b0 } };
          ilI <= 16'd0;
          l1I <= 1'b0;
          O0I <= { g_DDR_AXI_AWIDTH { 1'b0 } };
          O1I <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
          I1I <= 1'b0;
        end
      else
        begin
          case (OOl)
            i1I:
              begin
                o0I <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
                i0I <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
                l0I <= { g_DDR_AXI_AWIDTH { 1'b0 } };
                ilI <= 16'd0;
                l1I <= 1'b0;
                O1I <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
                I1I <= 1'b0;
                O0I <= O0I;
              end
            OoI:
              begin
                o0I <= video_resolution_i[(g_INPUT_X_W_RES_WIDTH - 1) : 0];
                i0I <= video_resolution_i[(g_INPUT_Y_H_RES_WIDTH + 16 - 1) : 16];
              end
            IoI:
              begin
                O1I <= i0I;
              end
            loI:
              begin
                if (IlI)
                  begin
                    I1I <= 1'b1;
                    l0I <= I0I;
                    ilI <= o0I * o1I * g_INITIAL_LINES_TO_BUFFER;
                  end
              end
            ooI:
              begin
                O0I <= l0I;
                l1I <= 1'b1;
              end
            ioI:
              begin
                if (read_ack_i)
                  begin
                    l0I <= l0I;
                    l1I <= 1'b0;
                    if (I1I)
                      begin
                        O1I <= O1I - g_INITIAL_LINES_TO_BUFFER;
                        I1I <= 1'b0;
                      end
                    else
                      begin
                        O1I <= O1I - g_SUBSEQUENT_LINES_TO_BUFFER;
                        I1I <= 1'b0;
                      end
                  end
                else
                  begin
                    l0I <= l0I;
                    ilI <= ilI;
                    l1I <= l1I;
                  end
              end
            OiI:
              begin
                if (read_done_i)
                  begin
                    l0I <= l0I + ilI;
                    ilI <= o0I * o1I * g_SUBSEQUENT_LINES_TO_BUFFER;
                  end
                else
                  begin
                    l0I <= l0I;
                    ilI <= ilI;
                  end
              end
            IiI:
              begin
                l0I <= l0I;
                ilI <= ilI;
                O1I <= O1I;
              end
            liI:
              begin
                l0I <= l0I;
                ilI <= ilI;
                O1I <= O1I;
              end
            oiI:
              begin
                l0I <= l0I;
                ilI <= ilI;
                l1I <= 1'b1;
              end
            iiI:
              begin
                l0I <= l0I;
                ilI <= ilI;
                l1I <= 1'b0;
              end
            default:
              begin
              end
          endcase
        end
    end
endmodule
